import net from 'node:net';
import http from 'node:http';
import { getStack } from '../utils/util.js';
import logger from '../utils/logger.js';
import { getDevServerURL } from '../middlewares/detect-dev-server.js';
export function createHttpProxy(target, port) {
    const server = http.createServer().listen(port, () => {
        logger.info(`Created http proxy on port ${port}`);
    });
    // default to 443, that's what browsers do
    const targetPort = target.match(/:(\d+)\//)?.[1] || '443';
    server.on('connect', (req, clientSocket, head) => {
        if (!req.url)
            return;
        const devServerUrl = getDevServerURL();
        if (!devServerUrl)
            return logger.info('Received connect event but dev server is not ready yet');
        const reqHost = req.headers.host?.split(':')[0] || '';
        const reqPort = req.headers.host?.split(':')[1] || '443';
        let serverHost;
        let serverPortStr;
        if (target.includes(reqHost)) {
            // intended appserver connection
            serverHost = devServerUrl.hostname;
            // do not change the port if not meant for the dev server
            serverPortStr = (reqPort === targetPort) ? devServerUrl.port : reqPort;
        }
        else {
            const url = new URL(`https://${req.url}`);
            serverHost = url.hostname;
            serverPortStr = url.port;
        }
        const serverPort = parseInt(serverPortStr, 10) || 443;
        logger.trace(`Http proxy: proxy connect from "${reqHost}:${reqPort}" to "${serverHost}:${serverPort}"`);
        const serverSocket = net.connect(serverPort, serverHost, () => {
            clientSocket.write('HTTP/1.1 200 Connection Established\r\n' +
                'Proxy-agent: Node-Proxy\r\n' +
                '\r\n');
            serverSocket.write(head);
            // reset timeout to default, because the connection now most likely works
            serverSocket.setTimeout(120000);
            serverSocket.pipe(clientSocket);
            clientSocket.pipe(serverSocket);
        });
        // add a quite low timeout value, because the proxy will always be reachable, but the remote might not
        serverSocket.setTimeout(10000);
        serverSocket.on('timeout', () => {
            serverSocket.end();
            clientSocket.end();
        });
        clientSocket.on('error', err => {
            logger.error(`Http proxy client socket error: ${getStack(err)}`);
        });
        serverSocket.on('error', err => {
            logger.error(`Http proxy server socket error: ${getStack(err)}`);
        });
    });
    server.on('clientError', err => {
        logger.error(`Http proxy clientError: ${getStack(err)}`);
    });
}
